#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Copyright (c) 2017 Luke Hinds <lukehinds@gmail.com>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0
##############################################################################

# from __future__ import division, print_function, absolute_import

"""Anteater - CI Gate Checks.

Usage:
  anteater (-p |--project) <project> [(-ps |--patchset) <patchset>]
  anteater (-p |--project) <project> [--path <project_path>]
  anteater (-h | --help)
  anteater --version

Options:
  -h --help     Show this screen.
  --version     Show version.
"""
from __future__ import absolute_import

import errno
import logging

import os
import six.moves.configparser
from docopt import docopt

from anteater import LOG
from anteater.src.patch_scan import prepare_patchset
from anteater.src.project_scan import prepare_project

config = six.moves.configparser.SafeConfigParser()
config.read('anteater.conf')
reports_dir = config.get('config', 'reports_dir')
__version__ = "0.11"
logger = logging.getLogger(__name__)


def _init_logging(anteater_log):
    """ Setup root logger for package """

    LOG.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s - %(name)s - '
                                  '%(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    ch.setLevel(logging.DEBUG)

    # create the directory if it does not exist
    path = os.path.dirname(anteater_log)
    try:
        os.makedirs(path)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

    handler = logging.FileHandler(anteater_log)
    handler.setFormatter(formatter)
    handler.setLevel(logging.DEBUG)
    del logging.root.handlers[:]
    logging.root.addHandler(ch)
    logging.root.addHandler(handler)


def check_dir():
    """ Creates a directory for scan reports """
    try:
        os.makedirs(reports_dir)
        logger.info('Creating reports directory: %s', reports_dir)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise


def main():
    """ Main function, mostly for passing arguments """
    _init_logging(config.get('config', 'anteater_log'))
    check_dir()
    arguments = docopt(__doc__, version=__version__)

    if arguments['<patchset>']:
        prepare_patchset(arguments['<project>'], arguments['<patchset>'])
    elif arguments['<project_path>']:
        prepare_project(arguments['<project>'], arguments['<project_path>'])


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Copyright (c) 2017 Luke Hinds <lukehinds@gmail.com>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0
##############################################################################

"""
    Accepts the --path argument and iterates the root directory using os.walk
    If a file is a binary, or contains a blacklisted string. If any violations
    are found, the script adds the violation to a log file.
"""

from __future__ import division, print_function, absolute_import
import hashlib
import six.moves.configparser
import os
import re
import logging
from binaryornot.check import is_binary

from . import get_lists

logger = logging.getLogger(__name__)
config = six.moves.configparser.SafeConfigParser()
config.read('anteater.conf')
anteater_files = config.get('config', 'anteater_files')
reports_dir = config.get('config', 'reports_dir')
ignore_dirs = ['.git', 'examples', anteater_files]
hasher = hashlib.sha256()


def prepare_project(project, project_dir):
    """ Generates blacklists / whitelists and calls main functions """

    # Get Various Lists / Project Waivers
    lists = get_lists.GetLists()

    # Get file name black list and project waivers
    file_audit_list, file_audit_project_list = lists.file_audit_list(project)

    # Get file content black list and project waivers
    flag_list, ignore_list = lists.file_content_list(project)

    # Get File Ignore Lists
    file_ignore = lists.file_ignore()

    # Perform rudimentary scans
    scan_file(project_dir, project, file_audit_list, file_audit_project_list,
              flag_list, ignore_list, file_ignore)


def scan_file(project_dir, project, file_audit_list, file_audit_project_list,
              flag_list, ignore_list, file_ignore):
    """Searches for banned strings and files that are listed """
    for root, dirs, files in os.walk(project_dir):
        # Filter out ignored directories from list.
        dirs[:] = [d for d in dirs if d not in ignore_dirs]
        for items in files:
            full_path = os.path.join(root, items)
            # Check for Blacklisted file names
            if file_audit_list.search(full_path) and not \
                    file_audit_project_list.search(full_path):
                match = file_audit_list.search(full_path)
                logger.error('Blacklisted filename: %s', full_path)
                logger.error('Matched String: %s', match.group())
                with open(reports_dir + "file-names_" + project + ".log",
                          "a") as gate_report:
                            gate_report. \
                                write('Blacklisted filename: {0}\n'.
                                      format(full_path))
                            gate_report. \
                                write('Matched String: {0}'.
                                      format(match.group()))

            # Check if Binary is whitelisted
            hashlist = get_lists.GetLists()
            split_path = full_path.split(project + '/', 1)[-1]
            binary_hash = hashlist.binary_hash(project, split_path)

            if is_binary(full_path):
                with open(full_path, 'rb') as afile:
                    buf = afile.read()
                    hasher.update(buf)
                if hasher.hexdigest() in binary_hash:
                    logger.info('Found matching file hash for: %s',
                                    full_path)
                else:
                    logger.error('Non Whitelisted Binary file: %s',
                                 full_path)
                    logger.error('Please submit patch with this hash: %s',
                                 hasher.hexdigest())
                    with open(reports_dir + "binaries-" + project + ".log",
                              "a") as gate_report:
                            gate_report.write('Non Whitelisted Binary: {0}\n'.
                                              format(full_path))

            else:
                if not items.endswith(tuple(file_ignore)):
                    try:
                        fo = open(full_path, 'r')
                        lines = fo.readlines()
                    except IOError:
                        logger.error('%s does not exist', full_path)

                    for line in lines:
                        # Check for sensitive content in project files
                        for key, value in flag_list.iteritems():
                            regex = value['regex']
                            desc = value['desc']
                            if re.search(regex, line) and not re.search(
                                    ignore_list, line):
                                logger.error('File contains violation: %s',
                                             full_path)
                                logger.error('Flagged Content: %s',
                                             line.rstrip())
                                logger.error('Matched Regular Exp: %s', regex)
                                logger.error('Rationale: %s', desc.rstrip())
                                with open(reports_dir + "contents-" + project
                                                  + ".log", "a") \
                                        as gate_report:
                                    gate_report. \
                                        write('File contains violation: {0}\n'.
                                              format(full_path))
                                    gate_report. \
                                        write('Flagged Content: {0}'.
                                              format(line))
                                    gate_report. \
                                        write('Matched Regular Exp: {0}'.
                                              format(regex))
                                    gate_report. \
                                        write('Rationale: {0}\n'.
                                              format(desc.rstrip()))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Copyright (c) 2017 Luke Hinds <lukehinds@gmail.com>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0
##############################################################################

"""
    Gathers various values from the gate check yaml file and return them to the
    calling instance
"""
from __future__ import absolute_import

import logging
import six.moves.configparser
import copy
import yaml
import re

config = six.moves.configparser.SafeConfigParser()
config.read('anteater.conf')
logger = logging.getLogger(__name__)
anteater_files = config.get('config', 'anteater_files')
flag_list = config.get('config', 'flag_list')
ignore_list = config.get('config', 'ignore_list')
ignore_dirs = ['.git', 'examples', anteater_files]

with open(flag_list, 'r') as f:
    fl = yaml.safe_load(f)

with open(ignore_list, 'r') as f:
    il = yaml.safe_load(f)


def _remove_nullvalue(contents):
    if contents and len(contents) > 2 and 'nullvalue' in contents:
        contents.remove('nullvalue')


def _merge(org, ded):
    ret = copy.deepcopy(org)
    for key in list(set([k for k in org] + [k for k in ded])):
        if key in org and key in ded:
            ret[key] = list(set(ret[key] + ded[key]))
            _remove_nullvalue(ret[key])
        elif key in ded:
            ret[key] = ded[key]
    return ret


class GetLists(object):
    def __init__(self, *args):
        # Placeholder for future args if more filters are needed
        self.args = args
        self.loaded = False

    def load_project_exception_file(self, project_exceptions, project):
        if self.loaded:
            return
        exception_file = None
        for item in project_exceptions:
            if project in item:
                exception_file = item.get(project)
        if exception_file is not None:
            with open(exception_file, 'r') as f:
                ex = yaml.safe_load(f)
            for key in ex:
                if key in fl:
                    fl[key][project] = _merge(fl[key][project], ex.get(key, None)) \
                            if project in fl[key] else ex.get(key, None)
            self.loaded = True

    def load_binary_exception_file(self, project_exceptions, project):
        if self.loaded:
            return
        exception_file = None
        for item in project_exceptions:
            if project in item:
                exception_file = item.get(project)
        if exception_file is not None:
            with open(exception_file, 'r') as f:
                ex = yaml.safe_load(f)
            for key in ex:
                if key in il:
                    il[key][project] = _merge(il[key][project], ex.get(key, None)) \
                            if project in il[key] else ex.get(key, None)
            self.loaded = True

    def binary_hash(self, project, patch_file):
        self.load_binary_exception_file(il.get('project_exceptions'), project)

        try:
            main_binary_hash = (il['binaries'][patch_file])
        except KeyError:
            main_binary_hash = []

        try:
            project_binary_hash = (il['binaries'][project][patch_file])
        except KeyError:
            project_binary_hash = []

        if main_binary_hash and project_binary_hash :
            logger.error('Warning: You have two hash entries for the %s file',
                         patch_file)
            logger.error('Check for duplicate entries in %s and %s.yaml for %s',
                         ignore_list, project, patch_file)

        new_list = main_binary_hash + project_binary_hash
        return new_list

    def file_audit_list(self, project):
        project_list = False
        self.load_project_exception_file(il.get('project_exceptions'), project)
        try:
            default_list = set((fl['file_audits']['file_names']))
        except KeyError:
            logger.error('Key Error processing file_names list values')
        try:
            project_list = set((fl['file_audits'][project]['file_names']))
            logger.info('file_names waivers found for %s', project)
        except KeyError:
            logger.info('No file_names waivers found for %s', project)

        file_names_re = re.compile("|".join(default_list),
                                   flags=re.IGNORECASE)

        if project_list:
            file_names_proj_re = re.compile("|".join(project_list),
                                            flags=re.IGNORECASE)
            return file_names_re, file_names_proj_re
        else:
            file_names_proj_re = re.compile("")
            return file_names_re, file_names_proj_re

    def file_content_list(self,  project):
        project_list = False
        self.load_project_exception_file(il.get('project_exceptions'), project)
        try:
            flag_list = (fl['file_audits']['file_contents'])

        except KeyError:
            logger.error('Key Error processing file_contents list values')

        try:
            ignore_list = il['file_audits']['file_contents']

        except KeyError:
            logger.error('Key Error processing file_contents list values')

        try:
            project_list = fl['file_audits'][project]['file_contents']

        except KeyError:
            logger.info('No file_contents waivers found  for %s', project)

        if project_list:
            ignore_list_merge = project_list + ignore_list

            ignore_list_re = re.compile("|".join(ignore_list_merge), flags=re.IGNORECASE)

            return flag_list, ignore_list_re
        else:
            ignore_list_re = re.compile("|".join(ignore_list),
                                        flags=re.IGNORECASE)
            return flag_list, ignore_list_re

    def file_ignore(self):
        try:
            file_ignore = (il['file_ignore'])
        except KeyError:
            logger.error('Key Error processing file_ignore list values')
        return file_ignore
